struct SRates 
{
    double mSL;
    double mTP;
    double mAsk;
    double mBid;
    double mSwap;
    double mOpen;
    double mClose;
    double mState;
    double mPoint;
    double mIndex;
    double mProfit;
    double mVolume;
    double mDirection;
    double mOpenSingle;
    double mVolumeSingle;
    datetime mTime;
    datetime mStart;
};

struct SSymbols
{
    string mName;
    SRates mRates[];
};

struct SOptions
{
    string mCurrency;
    double mBalance;
    double mContract;
    double mLeverage;
    double mStopOutLevel;
    SSymbols mSymbols[];
};

int PositionSelectVirtual(SOptions &options, const string symbol)
{
    int N = -1;
    int K = ArraySize(options.mSymbols);

    for (int k = 0; k < K; k++)
    {
        if (options.mSymbols[k].mName == symbol) 
        {
            N = k;
            break;
        }
    }
    
    if (N < 0)
    {
        N = K;
        ArrayResize(options.mSymbols, K + 1);
        options.mSymbols[K].mName = symbol;
    }
    
    return(N);
}

SRates PositionGetVirtual(SOptions &options, const string s)
{
    SRates D = {0};
    
    int digits = int(SymbolInfoInteger(s, SYMBOL_DIGITS));
    int K = PositionSelectVirtual(options, s);
    int N = ArraySize(options.mSymbols[K].mRates);

    D.mAsk = NormalizeDouble(SymbolInfoDouble(s, SYMBOL_ASK), digits);
    D.mBid = NormalizeDouble(SymbolInfoDouble(s, SYMBOL_BID), digits);

    if (N && options.mSymbols[K].mRates[N - 1].mState)
    {
        options.mSymbols[K].mRates[N - 1].mBid = D.mBid;
        options.mSymbols[K].mRates[N - 1].mAsk = D.mAsk;
    
        D = options.mSymbols[K].mRates[N - 1];

        D.mBid = NormalizeDouble(D.mTP && D.mTP <= D.mBid ? D.mTP : (D.mSL && D.mSL >= D.mBid ? D.mSL : D.mBid), digits);
        D.mAsk = NormalizeDouble(D.mTP && D.mTP >= D.mAsk ? D.mTP : (D.mSL && D.mSL <= D.mAsk ? D.mSL : D.mAsk), digits);
        D.mClose = NormalizeDouble(D.mDirection > 0 ? D.mBid : D.mAsk, digits);
        D.mPoint = NormalizeDouble((D.mClose - D.mOpen) * D.mDirection, digits);
        D.mProfit = NormalizeDouble(pipsToMoney(s, D.mVolume), 2);
        
        D.mSwap = 0;
        
        MqlDateTime day = {0};
        MqlDateTime deal = {0};
        
        TimeCurrent(day);
        TimeToStruct(D.mStart, deal);

        datetime current = StringToTime(
            string(day.year) + "." + 
            string(day.mon) + "." + 
            string(day.day));
            
        datetime start = StringToTime(
            string(deal.year) + "." + 
            string(deal.mon) + "." + 
            string(deal.day));

        while (current > start)
        {
            TimeToStruct(current, day);
            
            if (day.day_of_year != deal.day_of_year && 
                day.day_of_week != SATURDAY && 
                day.day_of_week != SUNDAY)
            {
                D.mSwap += NormalizeDouble(MathMax(D.mVolume, 1) * 
                    (SymbolInfoDouble(s, SYMBOL_TRADE_TICK_VALUE)) * 
                    (SymbolInfoDouble(s, D.mDirection > 0 ? SYMBOL_SWAP_LONG : SYMBOL_SWAP_SHORT)) *
                    (SymbolInfoInteger(s, SYMBOL_SWAP_ROLLOVER3DAYS) + 1 == day.day_of_week ? 3 : 1), 2);
            }
            
            current -= PeriodSeconds(PERIOD_D1);
        }
    }

    return(D);
}

SRates PositionOpenRelativeVirtual(
    SOptions &options, 
    const string symbol, 
    const int order, 
    const double volume, 
    const double SL = 0, 
    const double TP = 0,
    const double PL = 0)
{
    double ask = SymbolInfoDouble(symbol, SYMBOL_ASK);
    double bid = SymbolInfoDouble(symbol, SYMBOL_BID);
    double SLPrice = SL != 0 ? (order > 0 ? bid - SL : ask + SL) : 0;
    double TPPrice = TP != 0 ? (order > 0 ? ask + TP : bid - TP) : 0;
    double PLPrice = PL != 0 ? (order > 0 ? ask + PL : bid - PL) : 0;
    return(PositionOpenVirtual(options, symbol, order, volume, SLPrice, TPPrice));
}

SRates PositionOpenVirtual(
    SOptions &options, 
    const string symbol, 
    const double order, 
    const double volume, 
    const double SL = 0, 
    const double TP = 0)
{
    SRates P = {0};
    SRates D = PositionGetVirtual(options, symbol);
    int K = PositionSelectVirtual(options, symbol);
    int N = ArraySize(options.mSymbols[K].mRates);
    int digits = int(SymbolInfoInteger(symbol, SYMBOL_DIGITS));

    P.mState = 1;
    P.mIndex = N;
    P.mAsk = D.mAsk;
    P.mBid = D.mBid;
    P.mDirection = order;
    P.mStart = P.mTime = TimeCurrent();
    P.mSL = NormalizeDouble(SL, digits);
    P.mTP = NormalizeDouble(TP, digits);
    P.mVolume = P.mVolumeSingle = NormalizeDouble(volume, 2);
    P.mOpen = P.mOpenSingle = NormalizeDouble(P.mDirection > 0 ? D.mAsk : D.mBid, digits);

    if (D.mState)
    {
        bool addition = 
            P.mVolume > 0 && 
            D.mDirection == P.mDirection;

        P.mSwap = D.mSwap;
        P.mState = D.mState;
        P.mProfit = D.mProfit;
        P.mIndex = addition ? D.mIndex : P.mIndex;
        P.mStart = addition ? D.mStart : P.mStart;
        P.mSL = addition || P.mSL == 0 ? D.mSL : P.mSL;
        P.mTP = addition || P.mTP == 0 ? D.mTP : P.mTP;
        P.mDirection = addition ? D.mDirection : D.mDirection * -1;
        P.mClose = addition ? 0 : (D.mDirection > 0 ? D.mBid : D.mAsk);
        P.mVolume = MathAbs(addition ? D.mVolume + P.mVolume : D.mVolume - P.mVolume);
        
        if (P.mVolumeSingle)
        {
            P.mPoint = P.mVolumeSingle;
            P.mOpen = P.mOpenSingle * P.mVolumeSingle;

            for (int n = N - 1; n > -1; n--)
            {
                if (options.mSymbols[K].mRates[n].mClose || P.mStart > options.mSymbols[K].mRates[n].mTime) break;
                P.mOpen += options.mSymbols[K].mRates[n].mOpenSingle * options.mSymbols[K].mRates[n].mVolumeSingle;
                P.mPoint += options.mSymbols[K].mRates[n].mVolumeSingle;
            }

            P.mOpen = NormalizeDouble(P.mOpen / P.mPoint, digits);
            P.mPoint = NormalizeDouble((P.mClose - P.mOpen) * P.mDirection, digits);
        }
        
        options.mSymbols[K].mRates[N - 1].mState = 0;
    }

    if (P.mVolume == 0 || order == 0)
    {
        P.mState = 0;
    }

    if (GetMarginVirtual(options) < GetMarginDealVirtual(options, symbol, P.mVolumeSingle))
    {
        Print("No Margin");
        return(P);
    }

    ArrayResize(options.mSymbols[K].mRates, N + 1);
    options.mSymbols[K].mRates[N] = P;

    return(P);
}

void PositionCloseVirtual(SOptions &options, const string symbol)
{
    PositionOpenVirtual(options, symbol, 0, 0, 0, 0);
}

void PositionUpdateVirtual(SOptions &options, const string symbol, const double SL, const double TP)
{
    int index = PositionSelectVirtual(options, symbol);
    int digits = int(SymbolInfoInteger(symbol, SYMBOL_DIGITS));
    
    if (index > -1)
    {
        int N = ArraySize(options.mSymbols[index].mRates);
        
        if (N && options.mSymbols[index].mRates[N - 1].mState)
        {
            if (SL) options.mSymbols[index].mRates[N - 1].mSL = NormalizeDouble(SL, digits);
            if (TP) options.mSymbols[index].mRates[N - 1].mTP = NormalizeDouble(TP, digits);
        }
    }
}

void PositionTrailingVirtual(SOptions &options, const string symbol, const double step)
{
    SRates o = PositionGetVirtual(options, symbol);
    
    if (o.mState)
    {
        double point = SymbolInfoDouble(symbol, SYMBOL_POINT);
        double level = point * SymbolInfoInteger(symbol, SYMBOL_TRADE_STOPS_LEVEL);

        if (o.mDirection > 0)
        {
            o.mSL = o.mSL ? (o.mSL + step) : o.mBid - level - point;
            if (o.mBid - level >= o.mSL) PositionUpdateVirtual(options, symbol, o.mSL, o.mTP);
        }

        if (o.mDirection < 0)
        {
            o.mSL = o.mSL ? (o.mSL - step) : o.mAsk + level + point;
            if (o.mAsk + level <= o.mSL) PositionUpdateVirtual(options, symbol, o.mSL, o.mTP);
        }

        PositionControlVirtual(options, symbol);
    }
}

void PositionControlVirtual(SOptions &options, const string symbol)
{
    SRates o = PositionGetVirtual(options, symbol);
    
    if (o.mState)
    {
        bool C1 = o.mDirection > 0 && ((o.mSL && o.mBid <= o.mSL) || (o.mTP && o.mBid >= o.mTP));
        bool C2 = o.mDirection < 0 && ((o.mSL && o.mAsk >= o.mSL) || (o.mTP && o.mAsk <= o.mTP));
    
        if (C1 || C2)
        {
            PositionCloseVirtual(options, symbol);
        }
    }
}

double PositionTotalVirtual(SOptions &options)
{
    double summary = 0;
    int K = ArraySize(options.mSymbols);

    for (int k = 0; k < K; k++)
    {
        int N = ArraySize(options.mSymbols[k].mRates);
        summary += (N && options.mSymbols[k].mRates[N - 1].mState ? 1 : 0);
    }
    
    return(summary);
}

double CalculateProfitVirtual(SOptions &options, const string names, const double pips = 1)
{
    string syms[];
    double points = 0;
    double summary = 0;
    int K = StringSplit(names, ',', syms);

    for (int k = 0; k < K; k++)
    {
        SRates o = PositionGetVirtual(options, syms[k]);
        summary += o.mProfit + o.mSwap;
        points += o.mPoint;
    }
    
    return(pips ? points : summary);
}

double GetBalanceVirtual(SOptions &options, const int swaps = 0)
{
    double summary = 0;
    int K = ArraySize(options.mSymbols);
    
    for (int k = 0; k < K; k++)
    {
        int N = ArraySize(options.mSymbols[k].mRates);
        SRates o = PositionGetVirtual(options, options.mSymbols[k].mName);
        
        for (int n = 0; n < N; n++)
        {
            summary += options.mSymbols[k].mRates[n].mClose ? options.mSymbols[k].mRates[n].mProfit : 0;
            summary += swaps ? options.mSymbols[k].mRates[n].mSwap : 0;
        }
        
        summary += o.mProfit + (swaps ? o.mSwap : 0);
    }
    
    return(options.mBalance + summary);
}

double GetMarginDealVirtual(SOptions &options, const string symbol, const double volume)
{
    string parts[];
    int count = StringSplit(symbol, '.', parts);
    double contract = options.mContract * volume;
    string currency = StringSubstr(symbol, 0, 3) + options.mCurrency + (count ? "." + parts[1] : "");
    return(SymbolInfoDouble(currency, SYMBOL_LAST) * contract / options.mLeverage);
}

double GetMarginVirtual(SOptions &options)
{
    double summary = 0;
    int count = ArraySize(options.mSymbols);

    for (int k = 0; k < count; k++)
    {
        summary += CalculateProfitVirtual(options, options.mSymbols[k].mName, 0);
    }
    
    return(summary + options.mBalance);
}

double pipsToMoney(
  const string symbol, 
  const double pips,
  const int precision = 2)
{
  double step = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
  double value = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);
  return NormalizeDouble(pips * value / step, precision);
}

double moneyToPips(
  const string symbol, 
  const double amount,
  const int precision)
{
  double step = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
  double value = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);

  if (value == 0)
  {
    return 0;
  }

  return NormalizeDouble(amount * step / value, precision);
}